               Program DigitalDownConverter;

{********************************************************}
{*                                                      *}
{*             QRSS sound preprocessor                  *}
{*            By Alexander Yurkov RA9MB                 *}
{*                                                      *}
{********************************************************}


const
               pi = 3.1415926;
               bufmax = 30000;

var
               srate      : integer;  {Sampling rate}
               infreqmax  : real;     {Max Input freq}
               dt         : real;

               srrr       : integer;  {Sampling rate reduction ratio}
               cfreq      : real;     {Central freq}
               
               bw         : real;     {Bandwidth}
               lofreq     : real;     {Local oscillator freq}
               outfreqmax : real;     {Max out freq}
               outfreqmin : real;     {Min out freq}

               lo_re,
               lo_im      : double;   {Local oscillator voltage}
               lo_mult_re,
               lo_mult_im : double;   {Multiplier for one step of LO}
               
               filter_re,
               filter_im  : real;

               aufil,
               outfil     : file of word;     
             
               inbuf      : array [0..bufmax] of real;
               green_re,
               green_im   : array [0..bufmax] of real;
               
               snum       : longint;
               index      : word;
               minutes    : integer;
               cycles     : longint;
               gain       : real;       {IF gain}
               dispcy     : longint;    {display cycle}
               dispin     : longint;    {display index}
               insigmax, 
               outsigmax  : integer;
                
                Function SoundStreemReset
                   (sf : file of word)   : word; external;
                Function SoundStreemSetDuplex
                   (sf : file of word)   : word; external;   
                Function SoundStreemSync
                   (sf : file of word)   : word; external;
                Function SoundStreemSetRate
                   (sf : file of word, r : word)  : word; external;   
                Function SoundStreemS16LE
                   (sf : file of word)   : word; external;
                Function SoundStreemS16BE
                   (sf : file of word)   : word; external;
                Function SoundStreemSetChannels
                   (sf : file of word, chan : word)   : word; external;
                Function SoundStreemBufInfo
                   (sf : file of word)   : word; external;
                

               Procedure GetConfig;
var               
               config     : text;     {config file}
               i,
               znum       : integer;
               t,
               time       : real;

               begin
               Assign(config,'config.txt');
               ReSet(config);
               
               ReadLn(config,srate);
               dt:=1.0/srate;
               infreqmax:=srate*0.5;
               WriteLn('infreqmax: ',infreqmax);
               
               ReadLn(config,srrr);
               ReadLn(config,cfreq);
               bw:=(0.5*infreqmax)/srrr;
               WriteLn('bandwidth: ',bw);
               WriteLn('central freq:', cfreq);
               lofreq:=cfreq-bw;
               WriteLn('local oscilator freq: ',lofreq);
               outfreqmax:=cfreq+(0.5*bw);
               WriteLn('RXfreqmax: ',outfreqmax);
               outfreqmin:=cfreq-(0.5*bw);
               WriteLn('RXfreqmin: ',outfreqmin);
               if lofreq<0.0 then
                        begin
                        WriteLn('Central frequency is too low');
                        halt;
                        end;
               if outfreqmax>infreqmax then
                        begin
                        WriteLn('Central frequency is too high');
                        halt;
                        end;
               lo_mult_re:= Cos(2.0*pi*lofreq*dt*srrr);
               lo_mult_im:=-Sin(2.0*pi*lofreq*dt*srrr);
               lo_re:=1.0;
               lo_im:=0.0;
               
               ReadLn(config,znum);
               time:=znum/bw;
               snum:=2*Round(time/dt);
               if snum>bufmax then
                   begin
                   WriteLn('Filter time is too high');
                   end;
               for i:=0 to snum do
                   begin
                   t:=(i*dt)-time;
                   green_re[i]:=2.0*bw*dt;
                   if i<>(snum div 2) then
                            begin
                            green_re[i]:=green_re[i]*Sin(pi*t*bw);
                            green_re[i]:=green_re[i]/(pi*t*bw);
                            end;
                   green_im[i]:=-green_re[i]*Sin(2.0*pi*cfreq*t);
                   green_re[i]:=green_re[i]*Cos(2.0*pi*cfreq*t);
                   end;
               for i:=0 to bufmax do inbuf[i]:=0.0;
               index:=0;
               ReadLn(config,minutes);
               WriteLn('Minutes to work: ',minutes);
               cycles:=(minutes*60*srate) div srrr;
               ReadLn(config,gain);
               Close(config);
               dispcy:=srate div srrr;
               end;

var
               f    : text;
               il   : longint;
               i,n,m,
               j,k  : integer;
               iw   : word;
               
               t_re,
               t_im,
               sig  : real;
               isig : integer;
               wsig : word;
               buf  : word;
               
               
               begin
               WriteLn;
               WriteLn;
               GetConfig;
               {*** Debug out: ***}
                      {
               Assign(f,'green.txt');
               ReWrite(f);
               for i:=0 to snum do WriteLn(f,green_re[i],'   ',
                                   green_im[i],
                                   Sqrt(
                                   (green_re[i]*green_re[i])+
                                   (green_im[i]*green_im[i])
                                   ));
               Close(f);
                       }                              

               Assign(outfil,'digicnvout.raw');
               ReWrite(outfil);
               
               Assign(aufil,'/dev/dsp');
               ReSet(aufil);
               if 0<>SoundStreemReset(aufil) then
                   begin
                   WriteLn('Can not reset sound');
                   halt;
                   end;
               if 0<>SoundStreemSetChannels(aufil,1) then
                   begin
                   WriteLn('Can not set one channel');
                   halt;
                   end;
               if 0<>SoundStreemSetRate(aufil,srate) then
                   begin
                   WriteLn('Can not set rate ',srate);
                   halt;
                   end;
               if 0<>SoundStreemS16LE(aufil) then
                   begin
                   WriteLn('Can not set S16LE');
                   halt;
                   end;    
               if 0<>SoundStreemSync(aufil) then
                   begin
                   WriteLn('Can not sync');
                   halt;
                   end;
               
               {***** Main loop: *******}
               
               dispin:=0;
               WriteLn;
               WriteLn;
               
               for il:=0 to cycles do
                     begin
                     for j:=1 to srrr do
                         begin
                         
                         {*** Get signal ***}
                         
                         Read(aufil,k);
                         inbuf[index]:=k;
                         index := (index + 1) mod (bufmax+1);
                         if k>insigmax then insigmax:=k;
                         if k < -insigmax then insigmax:=-k;
                         
                         end;
                         
                     {*** Filtration ***}
                     
                     iw:=index;
                     filter_re:=0.0;
                     filter_im:=0.0;
                     for n:=0 to snum do
                         begin
                         filter_re:=filter_re + ( green_re[n] *
                                                  inbuf[iw]);
                         filter_im:=filter_im + ( green_im[n] *
                                                  inbuf[iw]);
                         iw := (iw+1) mod (bufmax+1);                         
                         end;
                     
                         
                     {*** Local oscillator ***}    

                     t_re:=(lo_re*lo_mult_re) - (lo_im*lo_mult_im);
                     t_im:=(lo_re*lo_mult_im) + (lo_im*lo_mult_re);
                     lo_re:=t_re;
                     lo_im:=t_im;
                     
                     {*** Out ***}
                     
                     sig := gain *
                         ((filter_re*lo_re) - (filter_im*lo_im));
                     if sig>30000.0 then sig:=32760;
                     if sig<-30000.0 then sig:=-32760;
                     isig:=Round(sig);
                     wsig:=$ffff and isig;
                     Write(outfil, wsig);
                     if isig >  outsigmax then outsigmax:=isig;                     
                     if isig < -outsigmax then outsigmax:=-isig;                     
                     
                     if dispin<dispcy then 
                         begin
                         dispin:=dispin+1;
                         end
                     else
                         begin
                         dispin:=0;
                         buf:=SoundStreemBufInfo(aufil);
                         WriteLn(Chr(27),Chr(77),
                         'input:',insigmax:6,
                         '   output:',outsigmax:6,
                         '   delaied bytes:',buf:6);
                         insigmax:=0;
                         outsigmax:=0;
                         
                         {*** LO amplitude stabilizator ***}
                         
                         sig:=1.0 + 0.5*( 
                             (lo_re*lo_re)+(lo_im*lo_im)-1.0 );
                         lo_re:=lo_re/sig;
                         lo_im:=lo_im/sig;    
                         
                         end;
                     end;
                     
               Close(outfil);
               
               end.      